/**
 * Jul 8, 2004, 3:47:17 PM
 * @author: F. MORON
 * @email: francois.moron@laposte.net
 * 
 * */
package org.exoplatform.container.client.http;

import java.util.ArrayList;
import java.util.List;
import java.util.regex.PatternSyntaxException;
//import net.sourceforge.wurfl.wurflapi.CapabilityMatrix;
//import net.sourceforge.wurfl.wurflapi.ObjectsManager;
//import net.sourceforge.wurfl.wurflapi.UAManager;
import org.dom4j.Document;
import org.dom4j.Node;
import org.dom4j.io.SAXReader;

public class ClientTypeMap {
  final static public String XHTML_MIME_TYPE = "text/xhtml" ;
  final static public String XHTMLMP_MIME_TYPE = "application/vnd.wap.xhtml+xml" ;

  private ArrayList clientList_;
  private static ClientTypeMap singleton_;
  
  private void loadClientsInfos() {
    try {
      ClassLoader cl = Thread.currentThread().getContextClassLoader() ;
      java.net.URL url = cl.getResource("conf/portal/clients-type.xml") ;
      SAXReader reader = new SAXReader();
      Document document = reader.read(url.openStream());
      List list = document.selectNodes("//clients-type/client-type");
      HttpClientType clientInfo;
      for(int i = 0; i <  list.size(); i++) {
        Node node = (Node) list.get(i) ;
        String name = node.selectSingleNode("name").getText() ;
        String userAgentPattern = node.selectSingleNode("userAgentPattern").getText() ;
        String preferredMimeType = node.selectSingleNode("preferredMimeType").getText() ;
        String renderer = node.selectSingleNode("type").getText() ;
        if (renderer.length() > 0) 
          clientInfo = new HttpClientType(name, userAgentPattern, preferredMimeType, renderer);
        else clientInfo = new HttpClientType(name, userAgentPattern, preferredMimeType);
        addClientInfo(clientInfo);
      } 
    } catch (Exception ex) {
      ex.printStackTrace() ;
    }
  }
  
  public ClientTypeMap() {
    clientList_ = new ArrayList() ;
    loadClientsInfos();
  }
  
  protected void addClientInfo(HttpClientType clientInfo) {
    clientList_.add(clientInfo);
  }
  
  /*
   * @return ClientInfo according to userAgent parameter and first
   * ClientInfo (ie5) if not found or if userAgent is null 
   * 
   */
  public HttpClientType findClient(String userAgent) {
    if (userAgent == null) return (HttpClientType) clientList_.get(0);
    if (userAgent.equals("")) return (HttpClientType) clientList_.get(0);
    HttpClientType client;
    for (int i = 0; i < clientList_.size(); i++) {
      client = (HttpClientType) clientList_.get(i);
      String userAgentPattern = client.getUserAgentPattern(); 
      if (userAgentPattern != null) {
        try {
          if (userAgent.matches(userAgentPattern)) return client;
        } catch (PatternSyntaxException e) {
          e.printStackTrace() ;
          return (HttpClientType) clientList_.get(0);
        }
      }
    }
    /*
    UAManager uam = ObjectsManager.getUAManagerInstance();
    String deviceID = uam.getDeviceIDFromUALoose(userAgent);
    if (!deviceID.equals("")) {
    	CapabilityMatrix cm = ObjectsManager.getCapabilityMatrixInstance();
    	String name = cm.getCapabilityForDevice(deviceID, "model_name");
    	if (cm.getCapabilityForDevice(deviceID, "preferred_markup").equals("html_wi_oma_xhtmlmp_1_0")) {
    		return new HttpClientType (name, deviceID, this.XHTMLMP_MIME_TYPE, 
    				HttpClientInfo.MOBILE_BROWSER_TYPE); 
    	}
    	//Other Mobile Markups : wml_1_1 wml_1_2 wml_1_3 -> wml not supported, standard browser returned (ie5)
    	//xhtml markup : html_wi_w3_xhtmlbasic
    }*/
    
    return (HttpClientType) clientList_.get(0);
  }
  
  public static ClientTypeMap getInstance() {
    if (singleton_ == null) {
      synchronized (ClientTypeMap.class) {
        singleton_ = new ClientTypeMap();
      }
    }
    return singleton_;
  }
}
