/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.services.scheduler.impl;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.ListIterator;
import java.util.Properties;

import org.exoplatform.container.PortalContainer;
import org.exoplatform.container.component.ComponentPlugin;
import org.exoplatform.container.xml.InitParams;
import org.exoplatform.services.scheduler.CronJob;
import org.exoplatform.services.scheduler.JobInfo;
import org.exoplatform.services.scheduler.JobSchedulerService;
import org.exoplatform.services.scheduler.PeriodInfo;
import org.exoplatform.services.scheduler.PeriodJob;
import org.picocontainer.Startable;
import org.quartz.CronTrigger;
import org.quartz.JobDetail;
import org.quartz.JobListener;
import org.quartz.Scheduler;
import org.quartz.SchedulerFactory;
import org.quartz.SimpleTrigger;
import org.quartz.Trigger;
import org.quartz.TriggerListener;
import org.quartz.impl.StdSchedulerFactory;
  
/**
 * Created by The eXo Platform SARL
 * Author : Hoa  Pham
 *          hoapham@exoplatform.com
 * Oct 5, 2005
 */
public class JobSchedulerServiceImpl implements JobSchedulerService, Startable {
  private Scheduler scheduler_ ;
  
  public JobSchedulerServiceImpl(InitParams params) throws Exception {
    /* TODO The following code allows to create as many Quartz Schedulers as
     * Portal Container instances. This prevents issues when Portal Containers
     * register jobs having the same description. It should be removed when
     * the Job Scheduler Service is moved to the RootContainer (JIRA EXOPL-21).
     */
    Properties props = new Properties();
    props.load(getClass().getClassLoader().getResourceAsStream
        ("org/exoplatform/services/scheduler/impl/quartz.properties"));
    props.setProperty(StdSchedulerFactory.PROP_SCHED_INSTANCE_NAME,
                      "job_scheduler_" + PortalContainer.getInstance().
                          getPortalContainerInfo().getContainerName());

    SchedulerFactory sf = new StdSchedulerFactory(props);
    scheduler_ = sf.getScheduler();
    scheduler_.start();
  }
  
  public void addJob(JobDetail job, Trigger trigger) throws Exception {
    scheduler_.scheduleJob(job,trigger);
  }
  
  public void addJob(JobInfo jinfo, Trigger trigger) throws Exception {
    JobDetail job=new JobDetail(jinfo.getJobName(), jinfo.getGroupName(),jinfo.getJob());
    scheduler_.scheduleJob(job,trigger);
  }
  
  public void addJob(JobInfo jinfo, Date date) throws Exception {
    SimpleTrigger trigger = new SimpleTrigger(jinfo.getJobName(), jinfo.getGroupName(),date);
    JobDetail job=new JobDetail(jinfo.getJobName(), jinfo.getGroupName(),jinfo.getJob());
    job.setDescription(jinfo.getDescription());
    scheduler_.scheduleJob(job,trigger);
  }

  public void addPeriodJob(JobInfo jinfo, int repeatCount, long period) throws Exception {
    int repeat;
    if(repeatCount<0) repeat=SimpleTrigger.REPEAT_INDEFINITELY ;
    else if (repeatCount==0) repeat=SimpleTrigger.REPEAT_INDEFINITELY ;
    else repeat=repeatCount-1;
    SimpleTrigger trigger = new SimpleTrigger(jinfo.getJobName(), 
                                              jinfo.getGroupName(),repeat,period);
    JobDetail job=new JobDetail(jinfo.getJobName(), jinfo.getGroupName(),jinfo.getJob());
    job.setDescription(jinfo.getDescription());
    scheduler_.scheduleJob(job,trigger);
  }
  
  public void addPeriodJob(JobInfo jinfo,PeriodInfo pinfo) throws Exception {
    int repeat = pinfo.getRepeatCount();
    Date start = pinfo.getStartTime();
    if(start == null) start = new Date();
    if(repeat<=0) repeat=SimpleTrigger.REPEAT_INDEFINITELY ;
    else repeat=repeat-1;
    SimpleTrigger trigger = new SimpleTrigger(jinfo.getJobName(), 
                                              jinfo.getGroupName(),
                                              start,pinfo.getEndTime(),
                                              repeat,pinfo.getRepeatInterval());
    JobDetail job=new JobDetail(jinfo.getJobName(), jinfo.getGroupName(),jinfo.getJob());
    job.setDescription(jinfo.getDescription());
    scheduler_.scheduleJob(job,trigger);
  }
  
  public void addPeriodJob(ComponentPlugin plugin)   throws Exception {
    PeriodJob pjob = (PeriodJob) plugin ;
    addPeriodJob(pjob.getJobInfo(), pjob.getPeriodInfo()) ;
  }

  public void addCronJob(JobInfo jinfo, String exp) throws Exception {
    CronTrigger trigger= new CronTrigger(jinfo.getJobName(), jinfo.getGroupName(),
                                         jinfo.getJobName(), jinfo.getGroupName(),exp) ;
    JobDetail job=new JobDetail(jinfo.getJobName(), jinfo.getGroupName(),jinfo.getJob());
    job.setDescription(jinfo.getDescription());
    scheduler_.addJob(job,true) ;
    scheduler_.scheduleJob(trigger) ;
  }
  
  public void addCronJob(ComponentPlugin plugin) throws Exception {
    CronJob cjob = (CronJob) plugin ;
    addCronJob(cjob.getJobInfo(),cjob.getExpression()) ;
  }
  
  public boolean removeJob(JobInfo jinfo) throws Exception {
    boolean b = scheduler_.deleteJob(jinfo.getJobName(), jinfo.getGroupName()) ;
    return b;
  }
  
  public List getAllExcutingJobs() throws Exception {
    List job;
    job = scheduler_.getCurrentlyExecutingJobs() ;
    return job;
  }
  
  public List getAllJobs() throws Exception {
    List jlist = new ArrayList();
    String jgroup [] = scheduler_.getJobGroupNames() ;
    for(int i=1 ; i<jgroup.length; i++) {
      String jname [] = scheduler_.getJobNames(jgroup[i]) ;
      for(int j = 0; j<jname.length; j++){
        jlist.add(scheduler_.getJobDetail(jname[j],jgroup[i]));
      }
    }
    return jlist;
  }
  
  public void addGlobalJobListener(ComponentPlugin plugin) throws Exception {
    scheduler_.addGlobalJobListener((JobListener) plugin) ;
  }
  
  public List getAllGlobalJobListener() throws Exception {
    List globalListener ;
    globalListener = scheduler_.getGlobalJobListeners();
    return globalListener;
  }
  
  public JobListener getGlobalJobListener(String name) throws Exception {
    List listener ;  
    JobListener jlistener ;
    listener =  scheduler_.getGlobalJobListeners() ;
    ListIterator iterator = listener.listIterator() ;
    while(iterator.hasNext()) {
      jlistener = (JobListener)iterator.next() ;
      if(jlistener.getName().equals(name)) {return jlistener ; }
    }
    return null ;
  }

  public boolean removeGlobalJobListener(String name) throws Exception {
    JobListener jlistener = getGlobalJobListener(name);
    boolean b = scheduler_.removeGlobalJobListener(jlistener) ;
    return b;
  }

  public void addGlobalTriggerListener(ComponentPlugin plugin)   throws Exception {
    scheduler_.addGlobalTriggerListener((TriggerListener) plugin) ;
  }
    
  public List getAllGlobalTriggerListener() throws Exception {
    List listener ;
    listener = scheduler_.getGlobalTriggerListeners() ;
    return listener;
  }
  
  public TriggerListener getGlobalTriggerListener(String name) throws Exception {
    TriggerListener tlistener ;
    List listener ;
    listener = scheduler_.getGlobalTriggerListeners() ;
    ListIterator iterator = listener.listIterator() ;
    while(iterator.hasNext()) {
      tlistener = (TriggerListener)iterator.next() ;
      if(tlistener.getName().equals(name)) { return tlistener ; }
    }
    return null;
  }

  public boolean removeGlobaTriggerListener(String name) throws Exception {
    TriggerListener tlistener = getGlobalTriggerListener(name) ;
    boolean b = scheduler_.removeGlobalTriggerListener(tlistener) ;
    return b;
  }

 
  public void addTriggerListener(ComponentPlugin plugin) throws Exception {   
  }

  public void start() { }

  public void stop() {  }  
}
