/***************************************************************************
 * Copyright 2001-2004 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
/**
 * Created by The eXo Platform SARL        .
 * Author : James Chamberlain
 *          james@echamberlains.com
 * Date: 11/2/2005
 * 
 */

package org.exoplatform.services.ldap.impl;


import java.util.HashMap;
import java.util.Hashtable;
import java.util.Map;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.ldap.InitialLdapContext;
import javax.naming.ldap.LdapContext;

import org.exoplatform.container.ExoContainer;
import org.exoplatform.container.component.ComponentPlugin;
import org.exoplatform.container.component.ComponentRequestLifecycle;
import org.exoplatform.container.xml.InitParams;
import org.exoplatform.services.ldap.CreateObjectCommand;
import org.exoplatform.services.ldap.DeleteObjectCommand;
import org.exoplatform.services.ldap.LDAPService;

public class LDAPServiceImpl implements LDAPService, ComponentRequestLifecycle {
  private ThreadLocal<LdapContext> tlocal_ = new ThreadLocal<LdapContext >() ;
  private Map<String, String> env = null;
  
  public LDAPServiceImpl(InitParams params) throws Exception {
    LDAPConnectionConfig config = 
      (LDAPConnectionConfig)params.getObjectParam("ldap.config").getObject() ;
    env = new HashMap<String, String>();
    env.put(Context.INITIAL_CONTEXT_FACTORY, "com.sun.jndi.ldap.LdapCtxFactory");
    env.put(Context.SECURITY_AUTHENTICATION, config.getAuthenticationType());
    env.put(Context.SECURITY_PRINCIPAL, config.getRootDN());
    env.put(Context.SECURITY_CREDENTIALS,  config.getPassword());
    env.put("com.sun.jndi.ldap.connect.timeout", "60000");
    env.put("com.sun.jndi.ldap.connect.pool", "true");
    env.put("java.naming.ldap.version", config.getVerion());
    env.put("java.naming.ldap.attributes.binary","tokenGroups");
    env.put(Context.PROVIDER_URL, config.getProviderURL());
    env.put(Context.REFERRAL, "follow");
  }
  
  
  public LdapContext getLdapContext() throws Exception {
    //new Exception("===================================").printStackTrace() ;
    LdapContext context = tlocal_.get() ;    
    if(context == null) {
      context = new InitialLdapContext(new Hashtable<String, String>(env), null) ;     
      tlocal_.set(context) ;
    } else {     
        context.setRequestControls(null);    
    }
    return context;
  }
  
  
  public InitialContext getInitialContext() throws Exception {
    Hashtable<String, String> props = new Hashtable<String, String>(env);
    props.put(Context.OBJECT_FACTORIES, "com.sun.jndi.ldap.obj.LdapGroupFactory");
    props.put(Context.STATE_FACTORIES, "com.sun.jndi.ldap.obj.LdapGroupFactory");
    return  new InitialLdapContext(props, null);
  }
  
  
  public boolean authenticate(String userDN, String password) throws Exception {
    Hashtable<String, String> props = new Hashtable<String, String>(env);
    props.put(Context.SECURITY_PRINCIPAL, userDN);
    props.put(Context.SECURITY_CREDENTIALS, password);
    props.remove("com.sun.jndi.ldap.connect.pool");
    props.put("com.sun.jndi.ldap.connect.pool", "false");
    new InitialLdapContext(props, null);
    return true;
  }
  
  public void addDeleteObject(ComponentPlugin plugin) throws Exception {
    DeleteObjectCommand command = (DeleteObjectCommand) plugin ;
    LdapContext ctx = getLdapContext() ;
    command.deleteObjects(ctx) ;
  }
  
  public void addCreateObject(ComponentPlugin plugin) throws Exception {
    CreateObjectCommand command = (CreateObjectCommand) plugin ;
    LdapContext ctx = getLdapContext() ;
    command.addObjects(ctx) ;
  }


  public void startRequest(ExoContainer container) {   }


  public void endRequest(ExoContainer container) {
    LdapContext context = tlocal_.get() ;
    if(context != null) {
      try {
        context.close() ;
        tlocal_.set(null);
      } catch(Exception ex) {
        ex.printStackTrace() ;
      }
    }
  }
}
