/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.services.organization;

import java.util.List;

import org.exoplatform.commons.utils.PageList;
import org.exoplatform.container.component.BaseComponentPlugin;
import org.exoplatform.container.xml.InitParams;
/**
 * Jul 20, 2004 
 * @author: Tuan Nguyen
 * @email:   tuan08@users.sourceforge.net
 * @version: $Id: OrganizationServiceListener.java,v 1.1 2004/07/21 18:15:46 tuan08 Exp $
 */
public class OrganizationDatabaseInitializer 
  extends BaseComponentPlugin implements OrganizationServiceInitializer {
  
  private OrganizationConfig config_ ;
 
  
  public OrganizationDatabaseInitializer(InitParams params) throws Exception {
    config_ = (OrganizationConfig) params.getObjectParamValues(OrganizationConfig.class).get(0) ;
  }
  
  public void init(OrganizationService service) throws Exception {
    PageList users = service.getUserHandler().getUserPageList(10);
    if(users != null && users.getAvailable() > 0) return ; //Not new database 
    createGroups(service) ;      
    createMembershipTypes(service) ;
    createUsers(service) ;
  }
  
  private void createGroups(OrganizationService orgService) throws Exception {
    List groups = config_.getGroup() ;
    for(int i = 0 ; i < groups.size() ; i++) {
      OrganizationConfig.Group data = (OrganizationConfig.Group) groups.get(i);
      Group group = orgService.getGroupHandler().createGroupInstance();
      group.setGroupName(data.getName());
      group.setDescription(data.getDescription()) ;
      String parentId = data.getParentId() ;
      if(parentId == null || parentId.length() == 0) {
        orgService.getGroupHandler().createGroup(group, true);
      } else {
        Group parentGroup =  orgService.getGroupHandler().findGroupById(parentId) ;
        orgService.getGroupHandler().addChild(parentGroup, group, true) ;
      }
    }
  }
  
  private void createMembershipTypes(OrganizationService service) throws Exception {
    List types = config_.getMembershipType() ;
    for(int i = 0 ; i < types.size() ; i++) {
      OrganizationConfig.MembershipType data = (OrganizationConfig.MembershipType) types.get(i);
      MembershipType type = service.getMembershipTypeHandler().createMembershipTypeInstance();
      type.setName(data.getType()) ; 
      type.setDescription(data.getDescription()) ;
      service.getMembershipTypeHandler().createMembershipType(type, true) ;
    }
  }
  
  private void createUsers(OrganizationService service) throws Exception {
    List users = config_.getUser() ;
    for(int i = 0 ; i < users.size() ; i++) {
      OrganizationConfig.User data = (OrganizationConfig.User) users.get(i);
      User user = service.getUserHandler().createUserInstance() ;
      user.setUserName(data.getUserName()) ;
      user.setPassword(data.getPassword()) ;
      user.setFirstName(data.getFirstName()) ;
      user.setLastName(data.getLastName()) ;
      user.setEmail(data.getEmail()) ;
      service.getUserHandler().createUser(user, true);
      String groups = data.getGroups();
      String[] entry = groups.split(",") ;
      for (int j = 0; j < entry.length; j++) {
        String[] temp = entry[j].trim().split(":");
        String membership = temp[0] ;
        String groupId = temp[1] ;
        Group group = service.getGroupHandler().findGroupById(groupId); 
        MembershipType mt = service.getMembershipTypeHandler().createMembershipTypeInstance() ;
        mt.setName( membership) ;
//      Membership m = service.getMembershipHandler().createMembershipInstance();
//      m.setMembershipType(membership);
        service.getMembershipHandler().linkMembership( user, group, mt, true);
      }            
    }
  }
}
