/**
 * Copyright 2001-2005 The eXo Platform SARL         All rights reserved.
 * Please look at license.txt in info directory for more license detail.
 **/

package org.exoplatform.services.organization.ldap;

import java.util.ArrayList;
import java.util.List;

import javax.naming.Name;
import javax.naming.NameParser;
import javax.naming.NamingEnumeration;
import javax.naming.directory.Attributes;
import javax.naming.directory.BasicAttribute;
import javax.naming.directory.BasicAttributes;
import javax.naming.directory.DirContext;
import javax.naming.directory.ModificationItem;
import javax.naming.directory.SearchControls;
import javax.naming.directory.SearchResult;
import javax.naming.ldap.Control;
import javax.naming.ldap.LdapContext;
import javax.naming.ldap.PagedResultsControl;

import org.exoplatform.commons.utils.ObjectPageList;
import org.exoplatform.commons.utils.PageList;
import org.exoplatform.services.ldap.LDAPService;
import org.exoplatform.services.organization.User;

import javax.naming.ldap.SortControl;



/**
 * Created by The eXo Platform SARL        .
 * Author : James Chamberlain
 *          james.chamberlain@gmail.com
*/

public class ActiveDirectoryUserHandlerImpl extends UserHandlerImpl {

	//	some useful constants from lmaccess.h
	private int UF_ACCOUNTDISABLE = 0x0002;
	private int UF_PASSWD_NOTREQD = 0x0020;
	private int UF_NORMAL_ACCOUNT = 0x0200;
	private int UF_PASSWORD_EXPIRED = 0x800000;
	
	public ActiveDirectoryUserHandlerImpl( LDAPAttributeMapping ldapAttrMapping, LDAPService ldapService) {
	  super(ldapAttrMapping, ldapService);
	}
	
	public void createUser(User user, boolean broadcast) throws Exception {		    
	  String userDN = "cn="+user.getUserName()+","+ldapAttrMapping_.userURL;			
	  Attributes attrs = ldapAttrMapping_.userToAttributes( user);    
	  // create userAccountControl - create the account disabled so it doesn't need a password
	  attrs.put("userAccountControl",Integer.toString(
	      UF_NORMAL_ACCOUNT + UF_PASSWD_NOTREQD + UF_PASSWORD_EXPIRED+ UF_ACCOUNTDISABLE));
    if(broadcast) preSave(user, true) ;
	  // Create user in directory
	  LdapContext ctx = ldapService_.getLdapContext();
	  ctx.createSubcontext( userDN, attrs);		
    if(broadcast) postSave(user, true) ;    
	} 
  
  public void saveUser( User user, boolean broadcast) throws Exception {
    LdapContext ctx = ldapService_.getLdapContext();    
    String userDN = this.getDNFromUsername(user.getUserName());
    if (userDN != null){
      User existingUser = this.findUserByDN(userDN, ctx);
      
      NameParser parser = ctx.getNameParser("");
      Name name = parser.parse(userDN);

      ArrayList<ModificationItem> modifications = new ArrayList<ModificationItem>();
      
      // update displayName & description
      if (!user.getFullName().equals(existingUser.getFullName())){
        ModificationItem mod = new ModificationItem(DirContext.REPLACE_ATTRIBUTE, 
            new BasicAttribute( ldapAttrMapping_.userDisplayNameAttr, user.getFullName()));
        modifications.add(mod);
        mod = new ModificationItem(DirContext.REPLACE_ATTRIBUTE, 
            new BasicAttribute( ldapAttrMapping_.ldapDescriptionAttr, "Account for "+user.getFullName()));
        modifications.add(mod);
      }
      
      // update account name
      if (!user.getUserName().equals(existingUser.getUserName())){
        ModificationItem mod = new ModificationItem(DirContext.REPLACE_ATTRIBUTE,
            new BasicAttribute( ldapAttrMapping_.userUsernameAttr, user.getUserName()));
        modifications.add(mod);
      }
      
      // update last name
      if (!user.getLastName().equals(existingUser.getLastName())){
        ModificationItem mod = new ModificationItem(DirContext.REPLACE_ATTRIBUTE, 
            new BasicAttribute( ldapAttrMapping_.userLastNameAttr, user.getLastName()));
        modifications.add(mod);
      }
      
      // update first name
      if (!user.getFirstName().equals(existingUser.getFirstName())){
        ModificationItem mod = new ModificationItem(DirContext.REPLACE_ATTRIBUTE, 
            new BasicAttribute( ldapAttrMapping_.userFirstNameAttr, user.getFirstName()));
        modifications.add(mod);
      }
      
      // update email
      if (!user.getEmail().equals(existingUser.getEmail())){
        ModificationItem mod = new ModificationItem(DirContext.REPLACE_ATTRIBUTE,
            new BasicAttribute( ldapAttrMapping_.userMailAttr, user.getEmail()));
        modifications.add(mod);
      }
      
      if (!user.getPassword().equals("PASSWORD")){
        ModificationItem mod = new ModificationItem(DirContext.REPLACE_ATTRIBUTE,
            new BasicAttribute( ldapAttrMapping_.userPassword, user.getPassword()));
        modifications.add(mod);
      }
      
      ModificationItem[] mods = new ModificationItem[ modifications.size()];      
      modifications.toArray(mods);
      if(broadcast) preSave(user, true);
      ctx.modifyAttributes(name, mods);
      if(broadcast) postSave(user, true);
    }
  }
  
//	private byte[] encryptPassword(String password) throws Exception {
//		byte passwordDigest[] = null;
//		String newQuotedPassword = "\"" + password + "\"";
//		passwordDigest = newQuotedPassword.getBytes("UTF-16LE");
//		return passwordDigest;
//	}
}
