package org.exoplatform.portlets.user.component;

import javax.faces.component.UIComponent;
import javax.faces.context.FacesContext;
import javax.faces.validator.Validator;
import javax.faces.validator.ValidatorException;
import org.exoplatform.faces.application.ExoFacesMessage;
import org.exoplatform.faces.core.component.* ;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.faces.core.validator.EmailAddressValidator;
import org.exoplatform.faces.core.validator.EmptyFieldValidator;
import org.exoplatform.container.PortalContainer ;
import org.exoplatform.portal.Information;
import org.exoplatform.services.organization.OrganizationService;
import org.exoplatform.services.organization.User;
/**
 * Sat, Jan 03, 2004 @ 11:16 
 * @author: Tuan Nguyen
 * @email:   tuan08@users.sourceforge.net
 * @version: $Id: UIAccountForm.java,v 1.14 2004/08/17 13:07:02 tuan08 Exp $
 */
public class UIAccountForm extends UISimpleForm {
  private static String USERNAME = "username" , PASSWORD1X = "password1x" ,
                        PASSWORD2X = "password2x" , EMAIL = "email" ,
                        FIRSTNAME = "firstname" , LASTNAME = "lastname" ;
  private User user_ ;
  
  public UIAccountForm() throws Exception {
    super("accountForm", "post", null);
    setRendererType("SimpleFormVelocityRenderer") ;
    setId("UIAccountForm") ;
    add(new UIStringInput(USERNAME, "").addValidator(EmptyFieldValidator.class)) ;
    add(new UIStringInput(PASSWORD1X, "").
        setType(UIStringInput.PASSWORD).
        addValidator(EmptyFieldValidator.class)) ;
    add(new UIStringInput(PASSWORD2X, "").
        setType(UIStringInput.PASSWORD).
        addValidator(EmptyFieldValidator.class)) ;
    add(new UIStringInput(FIRSTNAME, "").addValidator(EmptyFieldValidator.class)) ;
    add(new UIStringInput(LASTNAME, "").addValidator(EmptyFieldValidator.class)) ;
    add(new UIStringInput(EMAIL, "").addValidator(EmailAddressValidator.class)) ;
    addValidator(ConfirmPasswordValidator.class) ;
  }
   
  public void customizeNewAccountForm() {
    addActionListener(NewAccountListener.class, SAVE_ACTION) ;
  }
  
  public void customizeUpdateAccountForm() {
    addActionListener(UpdateAccountListener.class, SAVE_ACTION) ;
    getUIStringInput(USERNAME).setEditable(false);
  }
  
  public void setEditingUser(String userName) throws Exception {
    OrganizationService service =
      (OrganizationService) PortalContainer.getComponent(OrganizationService.class) ;
    user_ = service.getUserHandler().findUserByName(userName) ;
    setEditingUser(user_) ;
  }
  
  public void setEditingUser(User user) throws Exception {
    user_ = user;
    getUIStringInput(USERNAME).setValue(user_.getUserName());
    getUIStringInput(PASSWORD1X).setValue(user_.getPassword());
    getUIStringInput(PASSWORD2X).setValue(user_.getPassword());
    getUIStringInput(FIRSTNAME).setValue(user_.getFirstName());
    getUIStringInput(LASTNAME).setValue(user_.getLastName());
    getUIStringInput(EMAIL).setValue(user_.getEmail());
  }
  
  static public class ConfirmPasswordValidator implements Validator {
    public void validate(FacesContext context, UIComponent component,
                         Object value) throws ValidatorException {
      UIAccountForm uiForm = (UIAccountForm) component ;
      String p1 = uiForm.getUIStringInput(PASSWORD1X).getValue() ;
      String p2 = uiForm.getUIStringInput(PASSWORD2X).getValue() ;
      if(!p1.equals(p2)) {
        throw new ValidatorException(new ExoFacesMessage("#{UIAccountForm.msg.password-error}")) ;
      }
    }
  }
  
  static public class NewAccountListener extends ExoActionListener  {
    public void execute(ExoActionEvent event) throws Exception {
      UIAccountForm uicomp = (UIAccountForm) event.getComponent() ;    
      Information iprovider = findInformationProvider(uicomp);
      String userName = uicomp.getUIStringInput(USERNAME).getValue() ;
      OrganizationService service =
        (OrganizationService) PortalContainer.getComponent(OrganizationService.class) ;
      User newUser = service.getUserHandler().findUserByName(userName) ;
      if (newUser == null) {
        newUser = service.getUserHandler().createUserInstance() ;
        newUser.setUserName(userName) ;
        newUser.setPassword(uicomp.getUIStringInput(PASSWORD1X).getValue()) ;
        newUser.setFirstName(uicomp.getUIStringInput(FIRSTNAME).getValue()) ;
        newUser.setLastName(uicomp.getUIStringInput(LASTNAME).getValue()) ;
        newUser.setEmail(uicomp.getUIStringInput(EMAIL).getValue()) ;
        service.getUserHandler().createUser(newUser, true);
        uicomp.reset() ;
        iprovider.addMessage(new ExoFacesMessage("#{UIAccountForm.msg.create-user-success}")) ;
      } else {
        iprovider.addMessage(new ExoFacesMessage("#{UIAccountForm.msg.user-already-exist}")) ;
      }
    }
  }
  
  static public class UpdateAccountListener extends ExoActionListener  {
    public void execute(ExoActionEvent event) throws Exception {
      UIAccountForm  uicomp = (UIAccountForm) event.getComponent() ;
      uicomp.user_.setPassword(uicomp.getUIStringInput(PASSWORD1X).getValue()) ;
      uicomp.user_.setFirstName(uicomp.getUIStringInput(FIRSTNAME).getValue()) ;
      uicomp.user_.setLastName(uicomp.getUIStringInput(LASTNAME).getValue()) ;
      uicomp.user_.setEmail(uicomp.getUIStringInput(EMAIL).getValue()) ;
      OrganizationService service =
        (OrganizationService) PortalContainer.getComponent(OrganizationService.class) ;
      service.getUserHandler().saveUser(uicomp.user_, true) ;
      Information iprovider = findInformationProvider(uicomp);
      iprovider.addMessage(new ExoFacesMessage("#{UIAccountForm.msg.update-user-success}")) ;
    }
  }
}