/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.portlets.user.component;

import java.util.Collection;
import java.util.List;

import org.exoplatform.faces.core.component.UIExoCommand;
import org.exoplatform.faces.core.component.model.ListDataHandler;
import org.exoplatform.faces.core.component.model.Parameter;
import org.exoplatform.faces.core.event.CheckRoleInterceptor;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.portlets.user.component.UIOrganizationPortlet.UIMembershipNode;
import org.exoplatform.services.organization.OrganizationService;

/**
 * Sat, Jan 03, 2004 @ 11:16 
 * @author: Tuan Nguyen
 * @email:   tuan08@users.sourceforge.net
 * @version: $Id: UIListMembershipType.java,v 1.9 2004/08/26 04:18:02 tuan08 Exp $
 */
public class UIListMembershipType extends UIExoCommand {
  static Parameter[] EDIT_MEMBERSHIP = {new Parameter(ACTION , "editMembership") } ;
  static Parameter[] DELETE_MEMBERSHIP = {new Parameter(ACTION , "deleteMembership") } ;
  static Parameter[] ADD_MEMBERSHIP = {new Parameter(ACTION , "addMembership") };
  
  private List memberships_ ;
  private OrganizationService service_ ;
  private ListDataHandler dataHandler_ ;
  private boolean adminRole_ ;
   
  public UIListMembershipType(OrganizationService service) throws Exception {
    setId("UIListMembershipType") ;
    setRendererType("VelocityRenderer");
    service_ = service ;
    adminRole_ = hasRole("admin") ;
    dataHandler_ = new ListDataHandler() ;
    addActionListener(AddMembershipActionListener.class, "addMembership") ;
    addActionListener(EditMembershipActionListener.class, "editMembership") ;
    addActionListener(DeleteMembershipActionListener.class, "deleteMembership") ;
    update() ;
  }
  
  public void update() throws Exception {
    memberships_ =(List)service_.getMembershipTypeHandler().findMembershipTypes();
    dataHandler_.setDatas(memberships_) ;
  }
  
  public Parameter[] getDeleteParams() { return DELETE_MEMBERSHIP; };
  public Parameter[] getEditParams() { return EDIT_MEMBERSHIP; };
  public Parameter[] getAddParams() { return ADD_MEMBERSHIP; };
  
  public Collection getListMembership() throws Exception {
    return dataHandler_.getDatas();
  }
    
  public Collection getMembershipTypes() { return memberships_ ; }
  
  
  static public class AddMembershipActionListener extends ExoActionListener  {
    public AddMembershipActionListener() {
      addInterceptor(new CheckRoleInterceptor("admin")) ;  
    }
    
  	public void execute(ExoActionEvent event) throws Exception {
      UIListMembershipType uiList = (UIListMembershipType) event.getComponent() ;
  		UIMembershipNode uiController = (UIMembershipNode) uiList.getParent() ;
      UIMembershipTypeForm uiForm =  
      	(UIMembershipTypeForm)uiController.getChildComponentOfType(UIMembershipTypeForm.class) ;
      uiForm.addMembershipType() ;
      uiController.setRenderedComponent(uiForm.getId()) ;
    }
  }
  
  static public class EditMembershipActionListener extends ExoActionListener  {
    public EditMembershipActionListener() {
      addInterceptor(new CheckRoleInterceptor("admin")) ;  
    }
    
  	public void execute(ExoActionEvent event) throws Exception {
      UIListMembershipType uiList = (UIListMembershipType) event.getComponent() ;
  		String name = event.getParameter(OBJECTID) ;
      UIMembershipNode uiController = (UIMembershipNode) uiList.getParent() ;
      UIMembershipTypeForm uiForm = 
      	(UIMembershipTypeForm)uiController.getChildComponentOfType(UIMembershipTypeForm.class) ;
      uiForm.setMembershipType(name) ;
      uiController.setRenderedComponent(uiForm.getId()) ;
    }
  }
  
  static public class DeleteMembershipActionListener extends ExoActionListener  {
    public DeleteMembershipActionListener() {
      addInterceptor(new CheckRoleInterceptor("admin")) ;  
    }
    
  	public void execute(ExoActionEvent event) throws Exception {
      UIListMembershipType uiList = (UIListMembershipType) event.getComponent() ;
  		String name = event.getParameter(OBJECTID) ;
      uiList.service_.getMembershipTypeHandler().removeMembershipType(name, true) ;
      uiList.update() ;
    }
  }
}