/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.services.portal.impl;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.exoplatform.commons.utils.IOUtil;
import org.exoplatform.commons.utils.PageList;
import org.exoplatform.container.configuration.ConfigurationManager;
import org.exoplatform.container.xml.InitParams;
import org.exoplatform.services.cache.CacheService;
import org.exoplatform.services.cache.ExoCache;
import org.exoplatform.services.database.DBObjectPageList;
import org.exoplatform.services.database.HibernateService;
import org.exoplatform.services.database.ObjectQuery;
import org.exoplatform.services.portal.Interceptor;
import org.exoplatform.services.portal.PortalACL;
import org.exoplatform.services.portal.PortalConfigService;
import org.exoplatform.services.portal.Query;
import org.exoplatform.services.portal.model.Node;
import org.exoplatform.services.portal.model.Page;
import org.exoplatform.services.portal.model.PageSet;
import org.exoplatform.services.portal.model.PortalConfig;
import org.exoplatform.services.portletcontainer.pci.model.InitParam;
import org.hibernate.Session;
import org.jibx.runtime.BindingDirectory;
import org.jibx.runtime.IBindingFactory;
import org.jibx.runtime.IMarshallingContext;
import org.jibx.runtime.IUnmarshallingContext;
/**
 * Created by The eXo Platform SARL        .
 * Author : Tuan Nguyen
 *          tuan08@owners.sourceforge.net
 * Date: Jun 14, 2003
 * Time: 1:12:22 PM
 */
public class PortalConfigServiceImpl implements PortalConfigService {
  public static final String queryPageDataByOwner =
    "from pd in class org.exoplatform.services.portal.impl.PageData where pd.owner = ? ";
  public static final String queryNodeNavigationData =
    "from nv in class org.exoplatform.services.portal.impl.NodeNavigationData where nv.owner = ? ";
    
  private PortalACL portalACL_ ;
  private HibernateService hservice_ ;
  private Map predefinedTemplates_ ;
  private Interceptor interceptor_ ;
  private ExoCache portalConfigCache_ ;
  private ExoCache pageConfigCache_ ;
  private ExoCache nodeNavigationCache_ ;
  
  
  public PortalConfigServiceImpl(Interceptor interceptor, PortalACL portalACL,
                                 CacheService cacheService ,HibernateService hservice, 
                                 InitParams params, ConfigurationManager cservice) throws Exception {
    interceptor_ = interceptor ;
    hservice_ = hservice ;
    portalACL_ = portalACL ;
    portalConfigCache_ = cacheService.getCacheInstance(PortalConfig.class.getName()) ;
    pageConfigCache_ = cacheService.getCacheInstance(Page.class.getName()) ;
    nodeNavigationCache_ = cacheService.getCacheInstance(NodeImpl.class.getName()) ;
    String templateLocation = params.getValueParam("templateLocation").getValue() ;
    String xml = 
      IOUtil.getStreamContentAsString(cservice.getInputStream(templateLocation + "/templates.xml"));
    initPredefinedTemplates(xml) ;
  }
  
  public void initPredefinedTemplates(String xml) throws Exception {
  	predefinedTemplates_ = new HashMap() ;
    PageSet ps = (PageSet) unmarshall(new ByteArrayInputStream(xml.getBytes()), PageSet.class);
    List pages = ps.getPages() ;
    for (int i = 0 ; i <  pages.size(); i++) {
      Page page = (Page) pages.get(i) ;
      predefinedTemplates_.put(page.getOwner() + ":" + page.getName(), page) ;
    }
  }
  
  public void savePortalConfig(String owner, String xml)  throws Exception {
  	hservice_.save(new PortalConfigData(xml)) ;
    portalConfigCache_.remove(owner) ;
  }
  
  public void savePortalConfig(PortalConfig config)  throws Exception {
    portalConfigCache_.remove(config.getOwner()) ;
  	interceptor_.savePortalConfig(config) ;
  }
  
  public PortalConfig getPortalConfig(String owner) throws Exception {
    PortalConfig config = (PortalConfig) portalConfigCache_.get(owner) ;
    if(config != null) return config ;
    config =   interceptor_.getPortalConfig(owner);
    if(config != null) portalConfigCache_.put(owner, config) ;
    return config ;
  }
 
  public String getPortalConfigAsXmlString(String owner) throws Exception {
  	PortalConfigData  data = 
  		(PortalConfigData) hservice_.findOne(PortalConfigData.class, owner);
  	return data.getData() ;
  }

  public PageList findAllPortalConfigDescriptions(Query q) throws Exception {
  	String owner = q.getOwner() ;
  	if (owner == null || owner.length() == 0) {
  		owner = "%" ;
  	}
  	owner = owner.replace('*', '%') ;
    String viewPermission = q.getViewPermission() ; 
    String editPermission = q.getEditPermission() ; 
    ObjectQuery oq = new ObjectQuery(PortalConfigDescriptionData.class);
    oq.addLIKE("id", owner) ;
    if(viewPermission != null && viewPermission.length() > 0) {
      oq.addEQ("viewPermission", viewPermission) ;
    }
    if(editPermission != null && editPermission.length() > 0) {
      oq.addEQ("editPermission", editPermission) ;
    }
    return new DBObjectPageList(hservice_, oq) ;
  }
  
  public void removePortalConfig(String owner) throws Exception {
    hservice_.remove(PortalConfigData.class, owner) ;
    portalConfigCache_.remove(owner) ;
  }
  
  public void savePage(String xml)  throws Exception { 
    PageData data = new PageData(xml) ;
    hservice_.save(data) ;
    pageConfigCache_.remove(data.getId()) ;
  }

  public void savePage(Page page) throws Exception { 
    interceptor_.savePage(page) ;  
    pageConfigCache_.remove(PageData.getId(page)) ;
  }
  
  public Page getPage(String pageId) throws Exception {
    Page page = (Page)pageConfigCache_.get(pageId) ;
    if(page != null) return page ;
    page = interceptor_.getPage(pageId);
    pageConfigCache_.put(pageId, page) ;
    return page ;
  }

  public String getPageAsXmlString(String refId) throws Exception {
    PageData data = (PageData) hservice_.findOne(PageData.class, refId);
    return data.getData();
  }

  public void removePage(String refId) throws Exception {
    hservice_.remove(PageData.class, refId) ;
    pageConfigCache_.remove(refId) ;
  }

  public void removePageOfOwner(String owner) throws Exception {
  	Session session = hservice_.openSession();
  	
    List entries = session.createQuery(queryPageDataByOwner).setString(0, owner).list();
    for(int i=0; i<entries.size(); i++)
    	session.delete(entries.get(i));
  	session.flush() ;
    pageConfigCache_.clearCache() ;
  }
  
  public PageList findAllPageDescriptions(Query q) throws Exception {
  	String owner = q.getOwner() ;
  	if (owner == null || owner.length() == 0) {
  		owner = "%" ;
  	}
  	owner = owner.replace('*', '%') ;
    String viewPermission = q.getViewPermission() ; 
    String editPermission = q.getEditPermission() ; 
    ObjectQuery oq = new ObjectQuery(PageDescriptionData.class);
    oq.addLIKE("owner", owner) ;
    if(viewPermission != null && viewPermission.length() > 0) {
      oq.addEQ("viewPermission", viewPermission) ;
    }
    if(editPermission != null && editPermission.length() > 0) {
      oq.addEQ("editPermission", editPermission) ;
    }
    return new DBObjectPageList(hservice_, oq) ;
  }
  //=============================================================================
  public void saveNodeNavigation(String owner, Node node) throws Exception {
    interceptor_.saveNodeNavigation(owner, node) ;
    nodeNavigationCache_.remove(owner) ;
  }
  
  public void removeNodeNavigation(String owner) throws Exception  {
    hservice_.remove(NodeNavigationData.class, owner) ;
    nodeNavigationCache_.remove(owner) ;
  }
  
  public Node getNodeNavigation(String owner) throws Exception {
    Node node = (Node) nodeNavigationCache_.get(owner) ;
    if(node == null) {
      node =  interceptor_.getNodeNavigation(owner);
      nodeNavigationCache_.put(owner, node) ;
    }
    if(node == null) return null ;
    //clone the node
    return new NodeImpl(node, null, 0) ;
  }
  
  public Node createNodeInstance() { return new NodeImpl(); }
  //=============================================================================
  
  public PortalACL getPortalACL() { return portalACL_ ; }
  
  public List getPredefinedTemplates(String owner) {
  	ArrayList list = new ArrayList(predefinedTemplates_.size()) ;
  	Iterator i = predefinedTemplates_.values().iterator() ;
  	while(i.hasNext()) {
  		Page page = (Page)i.next() ;
  		if(owner.equals(page.getOwner())) {
  			list.add(page) ;
  		}
  	}
  	return list ;
  }
  
  public Page getPredefinedTemplate(String owner, String name) throws  Exception {
  	Page page = (Page)	predefinedTemplates_.get(owner + ":" + name)  ;
  	if (page != null) page = (Page)page.softCloneObject() ;
    return page ;
  }
  
  public void onModifiedOwner(String owner) throws Exception {
    portalConfigCache_.remove(owner) ;
    pageConfigCache_.clearCache() ;
    nodeNavigationCache_.clearCache() ;
  }
  
  public void onModifiedGroup(String groupId) throws Exception  {
    portalConfigCache_.clearCache() ;
    pageConfigCache_.clearCache() ;
    nodeNavigationCache_.clearCache();
  }
  
  
  static public Object unmarshall(InputStream is, Class type) throws Exception {
    IBindingFactory bfact = BindingDirectory.getFactory(type);
    IUnmarshallingContext uctx = bfact.createUnmarshallingContext();
    return uctx.unmarshalDocument(is, null);
  }
  
  static public void marshall( OutputStream os, Object obj) throws Exception {  
    IBindingFactory bfact = BindingDirectory.getFactory( obj.getClass());
    IMarshallingContext mctx = bfact.createMarshallingContext();
    mctx.setIndent(2);
    mctx.marshalDocument(obj, "UTF-8", null,  os) ;
  }
  
  static public String toXML(Object object) throws Exception {
    ByteArrayOutputStream os = new ByteArrayOutputStream() ;
    marshall(os , object) ;
    return new String(os.toByteArray()) ;
  }
  
  static public Object fromXML(String xml, Class type) throws Exception {
    ByteArrayInputStream is = new ByteArrayInputStream( xml.getBytes()) ;
    return unmarshall(is, type) ;
  }
}
