/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.services.portal.impl.converter.v11tov2;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.exoplatform.services.portal.model.Container;
import org.exoplatform.services.portal.model.Portlet;
import org.exoplatform.services.portal.model.PortletPreferences;
import org.exoplatform.services.portal.model.Preference;
import org.exoplatform.xml.object.XMLEntry;
import org.exoplatform.xml.object.XMLField;
import org.exoplatform.xml.object.XMLMap;
import org.exoplatform.xml.object.XMLObject;

/**
 * Created by The eXo Platform SARL
 * Author : Tuan Nguyen
 *          tuan08@users.sourceforge.net
 * Nov 24, 2005
 */
public class PortletConverter extends BaseConverter {
  
  List<Object> list = new ArrayList<Object>(); 
  
  public PortletConverter(){
    Object [][] obj = {
        { "windowId", "/content/DisplayStaticContent/banner", null},
        {"decorator", "*", "default"},
        {"renderer", "*", "default"},
        {"portletStyle", "*", "default"},      
        {"portletPreferences", 
          new String[]{"/portal/static-content/banner"}, 
          new String[]{"uri=war:/resources/community/web-content/banner.html"}}
    };
    list.add( obj);
    obj = new Object[][]{
        { "windowId",  
          "/nav/HorizontalMenu/toolbar", "/web/NavigationPortlet/community-toolbar"},
        {"decorator", "*", "default"},
        {"renderer", "*", "default"},
        {"portletStyle", "horizontal", "community-toolbar"},
        {"portletPreferences", null, new String[]{"community/navigation/horizontal/popup.vm"}}
    };
    list.add( obj);
    
    obj = new Object[][]{
        { "windowId", "/user/WelcomePortlet/welcome-user", null},
        {"title", "*", "Login"},
        {"width", "*", "325px"},
        {"portletStyle", "*", "community-login"}
    };
    list.add( obj);
    
    obj = new Object[][]{
        { "windowId", "/nav/Breadcrumbs/breadcrumbs", "/web/PortalInfoPortlet/breadcumbs"},  
        {"decorator", "*", "default"},
        {"renderer", "*", "default"},
        { "portletStyle", "breadcrumbs", "default"},
        { "height", "*", null},
        {"portletPreferences", null, new String[]{"community/breadcumbs.vm"}}
    };
    list.add( obj);
    
    obj = new Object[][]{
        { "windowId",  "/nav/Sitemap/sitemap", "/web/PortalInfoPortlet/sitemap"}, 
        {"decorator", "*", "default"},
        {"renderer", "*", "default"}, 
        { "portletStyle", "sitemap", "default"},        
        {"portletPreferences", null, new String[]{"community/sitemap.vm"}}
    };
    list.add( obj);
    
    obj = new Object[][]{
        { "windowId", "/content/DisplayContent/page1", "/content/DisplayStaticContent/intro"},  
          {"decorator", "*", "default"},
          {"renderer", "*", "default"}, 
          { "portletStyle", "*", "default"},       
          {"portletPreferences", new String[]{"title=Default Content","uri=/products/product"},
            new String[]{"title=Introduction","uri=war:/portal/static-content/home-intro"}}
    };
    list.add( obj);
    
    obj = new Object[][]{
        { "/exo-admin/JMXPortlet/jmx","/exo-admin/JMXPortlet/jmx", "/jmx/JMXPortlet/jmx"}};
    list.add( obj);
    
    obj = new Object[][]{
        { "windowId",  "/nav/Menu/menu", "/web/NavigationPortlet/menu"},        
        {"portletPreferences", null, 
          new String[]{"org/exoplatform/portlets/portal/renderer/html/vertical-navigation-popup.vm"}}
    };
    list.add( obj);
    
    obj = new Object[][]{
        { "windowId", "/exo-admin/SummaryPortlet/summary", "/web/PortalInfoPortlet/sitemap"},       
          { "portletStyle", "*", "default"},       
          {"portletPreferences", null, new String[]{"community/sitemap.vm"}}
    };
    list.add( obj);
    
    obj = new Object[][]{
        { "windowId", "/nav/VisitedPagesPortlet/visited-page", "/web/PortalInfoPortlet/visited-page"},       
          { "portletStyle", "*", "default"},       
          {"portletPreferences", null, new String[]{"community/visited-page.html"}}
    };
    list.add( obj);
    
    obj = new Object[][]{
        { "windowId", 
          "/exo-admin/PortletRegistery/registry", "/exo-admin/PortletManagement/portletmanagement"},       
          { "portletStyle", "*", "portletmanagement"},  
          {"title", "*", "Portlet Management"}       
    };
    list.add( obj);
    
    obj = new Object[][]{
        { "windowId", "/content/DisplayStaticContent/footer", null},         
          {"portletPreferences", new String[]{ "/portal/static-content/footer"}, 
            new String[]{"uri=war:/resources/community/web-content/footer.html"}}  
    };
    list.add( obj);
  }
  
  public void convert( XMLObject xml) throws Exception {       
    setField(xml, "showPortletMode", "*", false);  
    setField(xml, "showWindowState", "*", false); 
    setField(xml, "showInfoBar", "*", false);    
    for( int i = 0;  i<list.size(); i++){
      Object[][] o = (Object[][]) list.get(i);
      String id = String.valueOf( o[0][1]); 
      if(  xml.getFieldValue("windowId").toString().indexOf( id) > -1) {        
        for( int j = 0; j< o.length; j++) {
          setField( xml, (String) o[j][0], o[j][1], o[j][2]);
        }
        return;
      }
    }       
    setField( xml, "portletStyle", "*", "default");
    setField(xml, "decorator", "*", "default"); 
    setField(xml, "renderer", "*", "default");
    setField(xml, "portletPreferences", new String[]{}, new String[]{});  
  }
  
  public void pack( XMLObject xml) throws Exception {   
    Container pre = (Container)xml.getField("portalLayout").getObject().toObject();
    Container last = new Container();
    last.setDecorator( pre.getDecorator());
    last.setHeight( pre.getHeight());
    last.setId( pre.getId());
    last.setRenderer( pre.getRenderer());
    last.setTitle( pre.getTitle());
    last.setWidth( pre.getWidth());
    
    List list = new ArrayList();
    List children = pre.getChildren();
    
    Portlet p = findPortlet( children, "/content/DisplayStaticContent/banner");
    list.add( p);
    
    Container conToolbar = new Container();
    conToolbar.setDecorator( "community-toolbar");    
    conToolbar.setRenderer( "ContainerColumnRenderer");
    conToolbar.setId( null);
    List listToolbar = new ArrayList();
    p = findPortlet( children, "/web/NavigationPortlet/community-toolbar");
    if( p != null) listToolbar.add( p);
    p = findPortlet( children, "/user/WelcomePortlet/welcome-user");
    if( p != null) listToolbar.add( p);    
    conToolbar.setChildren( listToolbar);
    list.add( conToolbar);
    
    p = findPortlet( children, "/web/PortalModePortlet/portal-edit-mode");
    if( p == null){    
      p = new Portlet();
      p.setDecorator("default");
      p.setRenderer( "default");
      p.setShowInfoBar( false);
      p.setShowPortletMode( false);
      p.setShowWindowState( false);
      p.setPortletStyle("portal-mode");
      p.setId( null);
      p.setWindowId(xml.getFieldValue("owner")+":/web/PortalModePortlet/portal-edit-mode");
    }
    list.add( p);
    
    removeEmptyContainer( children);
    for( int i =0; i<children.size(); i++){
      list.add( children.get(i));
    }
    
    last.setChildren( list);
    xml.removeField( "portalLayout");
    xml.addField( "portalLayout", last.getClass(), last);    
  }
  private Portlet findPortlet( List list, String id) throws Exception{
    Portlet p = null;
    for( int i =0; i<list.size(); i++){
      if( list.get( i) instanceof Container) {
        p = findPortlet( ((Container)list.get( i)).getChildren(), id);        
      }else if ( list.get( i) instanceof Portlet){
        if( ((Portlet)list.get( i)).getWindowId().indexOf( id) > -1)
          p  = (Portlet)list.get( i);
      }
      if ( p != null){        
        list.remove( p);        
        break;
      }
    }
     return p;
  } 
  
  private void removeEmptyContainer( List list) throws Exception{  
    Portlet p = null;
    for( int i =0; i<list.size(); i++){
      if( list.get( i) instanceof Container) {
         Container con = (Container) list.get( i);
         if( con.getChildren().size() <= 0 ){
           list.remove( i);
           i--;
         }else{
           removeEmptyContainer( con.getChildren());
         }
      }
    }    
  } 
  
}
