/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.services.portal.log.impl;

import java.util.Date;
import java.util.GregorianCalendar;
import java.util.List;

import org.exoplatform.commons.utils.PageList;
import org.exoplatform.services.portal.log.PageLogData;
import org.exoplatform.services.portal.log.PortalActivitiesLogService;
import org.exoplatform.services.portal.log.PortalLogData;
import org.exoplatform.services.portal.log.PortletLogData;
import org.exoplatform.services.portal.log.Query;
import org.exoplatform.services.scheduler.JobInfo;
import org.exoplatform.services.scheduler.PeriodInfo;
import org.exoplatform.services.scheduler.JobSchedulerService;

/**
 * @author Hung Nguyen (nqhungvn@users.sourceforge.net)
 * @since Sep 26, 2005
 * @version $Id$
 */
public class PortalActivitiesLogServiceImpl implements PortalActivitiesLogService {
  private PortletActivitiesLogPlugin portletPlugin_ ;
  private PageActivitiesLogPlugin    pagePlugin_ ;
  private PortalActivitiesLogPlugin  portalPlugin_ ;
  private JobSchedulerService        jsService_ ;
  
  public PortalActivitiesLogServiceImpl(JobSchedulerService jsService) throws Exception {
    jsService_ = jsService ;
    GregorianCalendar calendar = new GregorianCalendar() ;
    int currentHour = calendar.get(GregorianCalendar.HOUR_OF_DAY) ;
    calendar.set(GregorianCalendar.HOUR_OF_DAY, currentHour + 1) ;
    calendar.set(GregorianCalendar.MINUTE, 0) ;
    calendar.set(GregorianCalendar.SECOND, 0) ;
    calendar.set(GregorianCalendar.MILLISECOND, 0) ; 
    Date runTaskDateTime = calendar.getTime() ;
    jsService_.addJob(new JobInfo(PeriodicComputeTimeSlotJob.class), runTaskDateTime) ;
    
  }
  
  public void setPortletLogActivitiesPlugin(PortletActivitiesLogPlugin plugin) {
    portletPlugin_ = plugin ;
  }
  public void setPageLogActivitiesPlugin(PageActivitiesLogPlugin plugin) {
    pagePlugin_ = plugin ;
  }
  public void setPortalLogActivitiesPlugin(PortalActivitiesLogPlugin plugin) {
    portalPlugin_ = plugin ;
  }
  final public void logPortletRender(String portletId, String portletAppId, 
      long exeTime, boolean error, String eventType) {
    portletPlugin_.logPortletRender(portletId, portletAppId, exeTime, error, eventType) ;
  }

  public void logPortletAction(String portletId, String portletAppId, long exeTime, boolean error, 
                               String eventType) {
	  portletPlugin_.logPortletAction(portletId, portletAppId, exeTime, error, eventType) ;
  }

  public void logPageAccess(String pageId, String pageOwner, long exeTime, boolean error) {
    pagePlugin_.logPageAccess(pageId, pageOwner, exeTime, error) ;
  }

  public void logPortalAccess(String portalId, String portalOwner, long exeTime, boolean error, boolean illegalAccess) {
    portalPlugin_.logPortalAccess(portalId, portalOwner, exeTime, error) ;
  }

  public PageList getPortletLogDatas(Query queryData) throws Exception {
    return portletPlugin_.getPortletLogDatas(queryData);
  }
  
  final public void savePortletLogData(PortletLogData data) throws Exception {
    portletPlugin_.savePortletLogData(data) ;
  }
  
  public List getPortletLogChartDatas(Query queryData) throws Exception {
    return portletPlugin_.getPortletLogChartDatas(queryData) ;
  }
  
  public PageList getPageLogDatas( Query queryData) throws Exception {
    return pagePlugin_.getPageLogDatas(queryData);
  }
  
  public List getPageLogChartDatas(Query queryData) throws Exception {
    return pagePlugin_.getPageLogChartDatas(queryData) ;
  }
  
  public void savePageLogData(PageLogData data) throws Exception {
    pagePlugin_.savePageLogData(data) ;
  }

  public PageList getPortalLogDatas(Query queryData) throws Exception {
    return portalPlugin_.getPortalLogDatas(queryData);
  }
  
  public List getPortalLogChartDatas(Query queryData) throws Exception {
    return portalPlugin_.getPortalLogChartDatas(queryData) ;
  }
  
  public void savePortalLogData(PortalLogData data) throws Exception {
    portalPlugin_.savePortalLogData(data) ;    
  }
  
  public void  saveMonitorData() throws Exception {
    portalPlugin_.saveMonitorData() ; 
    pagePlugin_.saveMonitorData();
    portletPlugin_.saveMonitorData() ;
  }
  
  public void startTimeSlotControl() throws Exception {
    PeriodInfo pInfo = new PeriodInfo( 0 , 3600*1000) ;
    jsService_.addPeriodJob(new JobInfo(PeriodicTimeSlotJob.class),pInfo) ;
  }
  
  public void updateTimeSlot() throws Exception {
    int portalPrevTimeIndex = portalPlugin_.getCurrentTimeIndex() ;
    int pagePrevTimeIndex = pagePlugin_.getCurrentTimeIndex() ;
    int portletPrevTimeIndex = portletPlugin_.getCurrentTimeIndex() ;
    portalPlugin_.updateTime() ;
    pagePlugin_.updateTime() ;
    portletPlugin_.updateTime() ;
    portalPlugin_.savePrevTimeSlotMonitorData(portalPrevTimeIndex) ;
    pagePlugin_.savePrevTimeSlotMonitorData(pagePrevTimeIndex) ;
    portletPlugin_.savePrevTimeSlotMonitorData(portletPrevTimeIndex) ;
    
  }
}
