/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.services.portal.test;

import java.io.File;
import java.util.Collection;
import java.util.Comparator;
import java.util.Enumeration;
import java.util.zip.ZipEntry;
import java.util.zip.ZipFile;

import org.exoplatform.container.PortalContainer;
import org.exoplatform.services.backup.ImportExportService;
import org.exoplatform.services.database.HibernateService;
import org.exoplatform.services.organization.Group;
import org.exoplatform.services.organization.Membership;
import org.exoplatform.services.organization.MembershipType;
import org.exoplatform.services.organization.OrganizationService;
import org.exoplatform.services.organization.User;
import org.exoplatform.services.organization.UserProfile;
import org.exoplatform.services.portal.impl.NodeNavigationData;
import org.exoplatform.services.portal.impl.PortalConfigData;
import org.exoplatform.services.portal.model.Page;
import org.exoplatform.test.BasicTestCase;

/**
 * Created by The eXo Platform SARL
 * Author : Tuan Nguyen
 *          tuan08@users.sourceforge.net
 * Dec 2, 2005
 */
abstract class ImportExportServiceTestBase extends BasicTestCase {
  
  protected static ImportExportService service_ ;
  protected static  OrganizationService orgService_ ;
  protected static HibernateService hservice_;
  static String defaultName  = "demo" ;
  static String Group1 = "Group1" ;
  static String Group2 = "Group2" ;
  static String uName_1 = "userName_1";
  static String uName_2 = "userName_2" ;
  static String mType_1 = "MembershipType_1" ;
  static String mType_2 = "MembershipType_2" ;   
    
  protected Group group1, group2, defaultGroup;  
  protected MembershipType mType1,mType2, defaultType ;
  protected User user1, user2 ,defaultUser; 
  
  protected boolean test  = true;
  
  public ImportExportServiceTestBase(String name) {
    super(name);
  }
  
  
  public void setUp() throws Exception {
    if( !test) return;
    PortalContainer manager  = PortalContainer.getInstance();
    service_ = (ImportExportService) manager.getComponentInstanceOfType(ImportExportService.class) ;
    orgService_ = (OrganizationService) manager.getComponentInstanceOfType(OrganizationService.class);
    hservice_ = (HibernateService) manager.getComponentInstanceOfType(HibernateService.class) ;    
  }
  
  public void tearDown() throws Exception {    
  }  
  
  public void prepareServiceData() throws Exception {
    defaultGroup = orgService_.getGroupHandler().findGroupById("/users") ; 
    Group g = orgService_.getGroupHandler().findGroupById("/"+Group1);
    if( g != null) orgService_.getGroupHandler().removeGroup( g, true);
    group1 = createGroup( Group1);      
   
    g = orgService_.getGroupHandler().findGroupById("/"+Group2);
    if( g != null) orgService_.getGroupHandler().removeGroup( g, true);
    group2 = createGroup( Group2) ;
    
    
    orgService_.getMembershipTypeHandler().removeMembershipType("member", true);
    defaultType = createMembershipType("member");
   
    
    orgService_.getMembershipTypeHandler().removeMembershipType( mType_1, true);
    mType1 = createMembershipType(mType_1); 
    
    
    orgService_.getMembershipTypeHandler().removeMembershipType( mType_2, true);
    mType2 = createMembershipType(mType_2);            
  }
  
  public void prepareUserData() throws Exception{
    prepareServiceData() ;
    
    orgService_.getUserHandler().removeUser( "demo", true);
    defaultUser = createUser("demo") ;
    createDataUser( defaultUser);
    
    orgService_.getUserHandler().removeUser(uName_1, true);
    user1 =  createUser(uName_1);
    createDataUser(user1);
   
    
    orgService_.getUserHandler().removeUser(uName_2, true);
    user2= createUser(uName_2) ;    
    createDataUser(user2) ;      
  }  
 
  public Group createGroup(String groupName) throws Exception {    
    Group groupParent = orgService_.getGroupHandler().createGroupInstance() ;
    groupParent.setGroupName( groupName);
    groupParent.setDescription("This is description");
    orgService_.getGroupHandler().createGroup( groupParent, true);   
    return groupParent;
  }
      
  public MembershipType createMembershipType(String name) throws Exception {
    MembershipType mt = orgService_.getMembershipTypeHandler().createMembershipTypeInstance();
    mt.setName( name) ;
    mt.setDescription("This is a test") ;
    mt.setOwner("exo") ;    
    orgService_.getMembershipTypeHandler().createMembershipType(mt, true);
    return mt;
  }
  
  public User createUser(String userName) throws Exception {   
    User user = orgService_.getUserHandler().createUserInstance() ;
    user.setUserName(userName) ;
    user.setPassword("default") ;
    user.setFirstName("default") ;
    user.setLastName("default") ;
    user.setEmail("exo@exoportal.org") ;
    orgService_.getUserHandler().createUser(user, true);
    return user ;
  }
  
  protected User createDataUser( User u) throws Exception {
    UserProfile up = orgService_.getUserProfileHandler().findUserProfileByName( u.getUserName());
    up.getUserInfoMap().put("user.gender", "male");
    orgService_.getUserProfileHandler().saveUserProfile(up, true);    
    return u;
  }
  
  protected ZipFile getZipServiceData(String serviceName) throws Exception{
    String zipFileName = service_.getServiceDataDirectory() + serviceName+ ".zip" ;
    ZipFile zipFile  = new ZipFile(new File(zipFileName)) ;
    return zipFile ;    
  }
   
  protected ZipFile getZipUserData(String userName) throws Exception{
    String zipFileName = service_.getUserDataDirectory() + userName+ ".zip" ;
    ZipFile zipFile  = new ZipFile(new File(zipFileName)) ;
    return zipFile ;    
  }
  
  protected boolean hasEntry(ZipFile zfile, String entryName) throws Exception {
    Enumeration e =  zfile.entries() ;
    while(e.hasMoreElements()) {
      ZipEntry entry = (ZipEntry) e.nextElement() ;
      if((entry.getName().endsWith(entryName))&&(entry.getSize()>0)) {return true ;}
     }    
    return false ;
  }
  
  protected boolean hasEntry(ZipFile zfile, String[] entryNames) throws Exception {
    Enumeration e =  zfile.entries() ;
    while(e.hasMoreElements()) {
      ZipEntry entry = (ZipEntry) e.nextElement() ;
      for(int i=0;i<entryNames.length;i++) {
        if((entry.getName().endsWith(entryNames[i]))&&(entry.getSize()>0)) {return true ; }
      }      
     }    
    return false ;
  }
  
  public static class GroupComparator implements Comparator {

    public int compare(Object o1, Object o2) {
      Group g1 = (Group) o1 ;
      Group g2 = (Group) o2 ;
      if(g1.getId().equals(g2.getId()))  return 0 ;
      return -1;
    }
  }
  
  public static class MembershipTypeComparator implements Comparator {

    public int compare(Object o1, Object o2) {
      MembershipType mType1 = (MembershipType) o1 ;
      MembershipType mType2 = (MembershipType) o2;
      if(mType1.getName().equals(mType2.getName())) return 0 ;  
      return -1;
    }    
  }  
  public static class MembershipComparator implements Comparator {

    public int compare(Object o1, Object o2) {
      Membership m1 = (Membership) o1 ;
      Membership m2 = (Membership) o2 ;      
      if ((m1.getGroupId().equals(m2.getGroupId()))
        &&(m1.getMembershipType().equals(m2.getMembershipType()))
        &&(m1.getUserName().equals(m2.getUserName()))
        ) {return 0 ;}
      return -1;
    }    
  }
  
  public void assertObject(Object o1, Object o2, Comparator comparator) throws Exception {
    if(comparator.compare(o1,o2)!=0) {throw new Exception ("Object "+ o1 + "not equals to" +o2) ;}  
  }
  
  public static class PortalConfigDataComparator implements Comparator {

    public int compare(Object o1, Object o2) {
      PortalConfigData p1 = (PortalConfigData) o1 ;
      PortalConfigData p2 = (PortalConfigData) o2 ;      
      if((p1.getOwner().equals(p2.getOwner()))
          &&(p1.getId().equals(p2.getId()))
          &&(p1.getEditPermission().equals(p2.getEditPermission()))
          &&(p1.getViewPermission().equals(p2.getViewPermission()))
          &&(p1.getPortalConfig().getDecorator().equals(p2.getPortalConfig().getDecorator()))
          &&(p1.getPortalConfig().getLocale().equals(p2.getPortalConfig().getLocale()))
          &&(p1.getPortalConfig().getRenderer().equals(p2.getPortalConfig().getRenderer()))
          ) { return 0 ; }
      return -1;
    }    
  }
  
  public static class PageConfigDataComparator implements Comparator {

    public int compare(Object o1, Object o2) {
      Page page1 = (Page) o1 ;
      Page page2 = (Page) o2 ;
      
      if(
          (page1.getEditPermission().equals(page2.getEditPermission()))
          &&(page1.getName().equals(page2.getName()))
          &&(page1.getOwner().equals(page2.getOwner()))
          &&(page1.getDecorator().equals(page2.getDecorator()))
          &&(page1.getTitle().equals(page2.getTitle()))
          &&(page1.getViewPermission().equals(page2.getViewPermission()))) { return 0 ; }
      return -1;
    }    
  }
  
  public static class NavigationConfigDataComparator implements Comparator {

    public int compare(Object o1, Object o2) {
      NodeNavigationData nnd1 = (NodeNavigationData) o1 ;
      NodeNavigationData nnd2 = (NodeNavigationData) o2 ;
      if((nnd1.getId().equals(nnd2.getId()))
          &&(nnd1.getOwner().equals(nnd2.getOwner()))
          &&(nnd1.getNodeNavigation().getNode().getName().equals(nnd2.getNodeNavigation().getNode().getName()))
          ) {return 0 ;}
      return -1;
    }
    
  }
}
